beam=25
symtab="data/lang/words.txt"
model="exp/mono/final.mdl"
graphdir="exp/mono/graph"
db_name="dev"
epoch=$1
# File with dev set scores.
pred_file=$2"_ep_"$epoch"."$db_name".scores"

echo "Decoding parameters: "
echo "	db_name = " $db_name
echo "	prediction file = "$pred_file
echo "	epoch = " $epoch


outdir="exp/mono/scores/"$db_name
trans_file=$outdir/$db_name.tra
ali_file=$outdir/$db_name.ali

ref_file="data/"$db_name/text

mkdir -p $outdir
rm -f tmpScore
mapping="en:n,ao:aa,ax-h:ah,ax:ah,ix:ih,el:l,zh:sh,ux:uw,axr:er,em:m,nx:n,eng:ng,hv:hh,pcl:pau,tcl:pau,kcl:pau,q:pau,bcl:pau,dcl:pau,gcl:pau,epi:pau"
for acousticscale in 1 .9 .8 .7 .6 .5  ; do
   cat $pred_file | decode-faster-mapped --beam=$beam \
                       --acoustic-scale=$acousticscale \
                       --word-symbol-table=$symtab $model \
                       $graphdir/HCLG.fst ark,t:- ark,t:$trans_file \
                       ark,t:$ali_file 2>> $outdir.log

   scripts/collapse_phones.pl --ignore-first-field \
                       $symtab $mapping < $trans_file > tmp
 
   score=$(scripts/sym2int.pl --ignore-first-field $symtab $ref_file | \
        scripts/collapse_phones.pl --ignore-first-field $symtab $mapping |\
        compute-wer --mode=present ark:-  ark,p:tmp 2>/dev/null | grep "%WER" |\
                           awk '{print $2; }')
   echo "Dev set acoustic scale =" $acousticscale " err = " $score
   echo $acousticscale " " $score >> tmpScore
done
rm tmp

#acousticscale=$(cat tmpScore  | perl -ane '$minacc = 1000000 ; $scale =1; while (<STDIN>) { chomp ; @pieces = split(" "); if ($pieces[1] < $minacc) { $minacc = $pieces[1] ; $scale = $pieces[0] ; } } ; print $scale;')
acousticscale=$(cat tmpScore  | perl min_score.pl)
rm tmpScore 

echo "Acoustic scale for coretest = " $acousticscale
db_name="test"
# File with core test scores.
pred_file=$2"_ep_"$epoch".core.scores"
outdir="exp/mono/scores/"$db_name
trans_file=$outdir/$db_name.tra
ali_file=$outdir/$db_name.ali

ref_file="data/"$db_name/text

mkdir -p $outdir
cat $pred_file | decode-faster-mapped --beam=$beam \
                   --acoustic-scale=$acousticscale --word-symbol-table=$symtab \
                   $model $graphdir/HCLG.fst ark,t:- ark,t:$trans_file \
                   ark,t:$ali_file 2>> $outdir.log

echo "CORE TEST RESULTS"
scripts/collapse_phones.pl --ignore-first-field \
                    $symtab $mapping < $trans_file > tmp
scripts/sym2int.pl --ignore-first-field $symtab $ref_file | \
     scripts/collapse_phones.pl --ignore-first-field $symtab $mapping |\
     compute-wer --mode=present ark:-  ark,p:tmp 2>/dev/null 
 
